//============================================================================
//  aPCI-8102 Device Driver I/F Control
//  apci8102ctl.c
//  Author    K. Fujimaki(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date      2001/07/16
//  Rev.      1.0.0
//============================================================================

#include <stdio.h>	// used printf,perror...
#include <pthread.h>	// used pthread
#include <fcntl.h>	// used open
#include <unistd.h>	// used close,usleep
#include <sys/ioctl.h>	// used ioctl
#include <sys/poll.h>	// used poll
#include <string.h>	// used strerror
#include <errno.h>	// used errno

#include "apci8102ctl.h"


/*----------------- Define Device Control ----------------------------------*/

#define APCI8102_FNC_EXEC	1		// ¹
#define APCI8102_FNC_NOTEXEC	0		// ¹Բǽ

typedef struct _apci8102_ctl
{
	int	  fd;
	void*	  arg;
	int	  bexec;			// ǽ¹
} apci8102_ctl, *papci8102_ctl;

static apci8102_ctl ctl[APCI8102_MAX_DEVICES] =
{
	{ 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 },
	{ 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }
};


/*----------------- Define Static Functions --------------------------------*/

static int	dev_open(int log_slot);
static int	is_log_slot(int log_slot);
static int	is_pass(int log_slot);


/*==========================================================================*/
/*  apci8102_create - ǥХץ					    */
/*  Input : log_slot - åȻ(APCI8102_SLOT_AUTO ξ硢	    */
/*					̤ѥǥХ)		    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 ʾ å				    */
/*==========================================================================*/
int apci8102_create(int log_slot)
{
	int fd = 0, i;

	// åȻ
	if (log_slot != APCI8102_SLOT_AUTO)
	{
		// åȤ
		if (!(log_slot >= 0 && log_slot < APCI8102_MAX_DEVICES))
			return APCI8102_EINVAL;

		// ǥХץ
		fd = dev_open(log_slot);
		if (fd < 0)
		{
			PDEBUG("Error: open, %s\n", strerror(errno));

			if (errno == EBUSY)
				return APCI8102_EBUSY;
			return APCI8102_ENODEV;
		}

		ctl[log_slot].fd    = fd;
		ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;

		return log_slot;
	}

	// ̤ѥǥХץ
	for (i = 0; i < APCI8102_MAX_DEVICES; i++)
	{
		// ǥХϥץ
		if (ctl[i].fd != 0)
			continue;

		// ǥХץ
		fd = dev_open(i);
		if (fd < 0)
			continue;

		break;
	}

	// ץǤǥХ̵
	if (i >= APCI8102_MAX_DEVICES)
		return APCI8102_ENODEV;

	ctl[i].fd    = fd;
	ctl[i].bexec = APCI8102_FNC_NOTEXEC;

	return i;		// return log_slot
}

/*==========================================================================*/
/*  apci8102_close - ǥХ					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 						    */
/*==========================================================================*/
int apci8102_close(int log_slot)
{
	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8102_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8102_EBUSY;

	ctl[log_slot].bexec = APCI8102_FNC_EXEC;		// ¹

	// ǥХΥ
	close(ctl[log_slot].fd);

	ctl[log_slot].fd    = 0;
	ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8102_OK;
}

/*==========================================================================*/
/*  apci8102_get_dirs - ǥ쥯			    */
/*  Input : log_slot - åȻ					    */
/*  pdirs - ǥ쥯Ǽΰ					    */
/*  Output: return - 0 ̤ 顼				    */
/*  		     0 						    */
/*==========================================================================*/
int apci8102_get_dirs(int log_slot, unsigned char* pdirs)
{
	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8102_EINVAL;

	if (pdirs == NULL)
		return APCI8102_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8102_EBUSY;

	ctl[log_slot].bexec = APCI8102_FNC_EXEC;		// ¹

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8102_GET_DIRS, pdirs) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8102_ESYSCALL;
	}

	ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8102_OK;
}

/*==========================================================================*/
/*  apci8102_set_dirs - ǥ쥯				    */
/*  Input : log_slot - åȻ					    */
/*  	    dirs - ǥ쥯					    */
/*  Output: return - 0 ̤ 顼				    */
/*  		     0 						    */
/*==========================================================================*/
int apci8102_set_dirs(int log_slot, unsigned char dirs)
{
	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8102_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8102_EBUSY;

	ctl[log_slot].bexec = APCI8102_FNC_EXEC;		// ¹

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8102_SET_DIRS, &dirs) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8102_ESYSCALL;
	}

	ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8102_OK;
}

/*==========================================================================*/
/*  apci8102_inp - ϥݡȤǡ				    */
/*  Input : log_slot - åȻ					    */
/*  	    port  - ݡȻ						    */
/*  	    pdata - ǡǼΰ					    */
/*  Output: return - 0 ̤ 顼				    */
/*  	 	     0 						    */
/*==========================================================================*/
int apci8102_inp(int log_slot, int port, unsigned char* pdata)
{
	apci8102_io iobuf;

	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8102_EINVAL;

	if (pdata == NULL)
		return APCI8102_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8102_EBUSY;

	ctl[log_slot].bexec = APCI8102_FNC_EXEC;		// ¹

	// ѥ᡼
	iobuf.port = port;

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8102_IN_PORT, &iobuf) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8102_ESYSCALL;
	}

	*pdata = iobuf.value;

	ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8102_OK;
}

/*==========================================================================*/
/*  apci8102_outp - ϥݡȤإǡ				    */
/*  Input : log_slot - åȻ					    */
/*  	    port - ݡȻ						    */
/*  	    data - ϥǡ						    */
/*  Output: return - 0 ̤ 顼				    */
/*  		     0 						    */
/*==========================================================================*/
int apci8102_outp(int log_slot, int port, unsigned char data)
{
	apci8102_io iobuf;

	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8102_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8102_EBUSY;

	ctl[log_slot].bexec = APCI8102_FNC_EXEC;		// ¹

	// ѥ᡼
	iobuf.port	= port;
	iobuf.value = data;

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8102_OUT_PORT, &iobuf) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8102_ESYSCALL;
	}

	ctl[log_slot].bexec = APCI8102_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8102_OK;
}

/*--------------------------------------------------------------------------*/
/*  dev_open - ǥХΥץ					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - open 					    */
/*--------------------------------------------------------------------------*/
static int dev_open(int log_slot)
{
	char szbuf[30];

	sprintf(szbuf, "/dev/apci8102%d", log_slot);

	return open(szbuf, O_RDWR);
}

/*--------------------------------------------------------------------------*/
/*  is_log_slot - åȥå					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̵						    */
/*		     1 						    */
/*--------------------------------------------------------------------------*/
static int is_log_slot(int log_slot)
{
	// ϰϤ
	if (!(log_slot >= 0 && log_slot < APCI8102_MAX_DEVICES))
		return 0;

	// ǥХξ
	if (ctl[log_slot].fd == 0)
		return 0;

	// Ѳǽ
	return 1;
}

/*--------------------------------------------------------------------------*/
/*  is_pass - ¹Բǽå						    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̵						    */
/*		     1 						    */
/*--------------------------------------------------------------------------*/
static int is_pass(int log_slot)
{
	// ǽμ¹Ծ
	if (ctl[log_slot].bexec != APCI8102_FNC_NOTEXEC)
		return 0;	// ¹Բ

	// ¹Բǽ
	return 1;
}



